@echo off
cls
REM ************************************************************************************
REM Description	: ARCADE (MAME with ui [formerly MAMEFX]) updating/compiling batch-file
REM Author		: Mucci
REM Date		: 2016-04-01
REM
REM Used external programs (is already in the zip file)
REM				 - Notepad2.exe from http://www.flos-freeware.ch/
REM
REM Changes		: 1.0 first version
REM Changes		: 1.0a get version from %~dp0\store1\src\osd\winui\winui.h
REM ************************************************************************************

REM This script is used only for the MAMEdev tools
REM Place this script and Notepad2.exe to your \src\ folder
REM Start always conemu.bat first (Environment) and after that "compile-ARCADE.cmd"

REM Check on which OS is runing this script (x32/x64) and set the variable
if defined PROGRAMFILES(X86) (
    set CompileX=x64
) else (
    set CompileX=x86
)
REM set the value for Architecture: x86 or x64 (default=x64) -> if you want to override the autotetection
REM set CompileX=x64

REM Set the default values for some settings
REM
REM define version to compile as default = MAME or ARCADE (default=ARCADE)
set Compileversion=ARCADE
REM CPU Core -> change that to the amount of cpu-core you have and add 1 -> "core +1"
REM Example: dual-core or quad-core processor -> use the option -j3 or -j5
set Compilecpu=j13
REM Compile with tools: 0/off/without or 1/on/with (default=0/off/without)
set Compiletools=0
set Compiletoolsstring=off
set Compiletoolsstring2=without
REM set default values for pause: on or off (default=on)
set batchpause=on
REM set default values for doing a clean before compiling: on or off (default=off)
set cleanmode=off
REM set default value for addon folder
set Addonfolder=off

REM Displaying a start information
ECHO    -----------------------------------------------
ECHO      ARCADE update/compiling batch-file starting
ECHO    -----------------------------------------------
ECHO         (c) 2016 mameguide.info
ECHO.

REM Check if ARCADE source is installed or not
IF EXIST %~dp0\store1\ GOTO JM-checkvar
ECHO    ---------------------------------------
ECHO      downloading ARCADE Source
ECHO    ---------------------------------------
ECHO.
REM Copy the source from ARCADE using the git command
git clone https://github.com/Robbbert/store1.git
cd %~dp0\store1\
REM Set Build variable
set /p Build=<%~dp0\store1\.git\refs\heads\master
REM Write found build numbers to a file and convert it to the revision number
git rev-list %Build% --count >%~dp0\store1\revision.txt
REM Write Version number to a file
FINDSTR /C:"#define MAME_VERSION" %~dp0\store1\src\osd\winui\winui.h >version_raw.txt
set /p MAMEVERSION=<version_raw.txt
set MAMEVERSION=%MAMEVERSION:~23,-1%
echo.%MAMEVERSION%>version.txt
set /p MAMEVERSION=<%~dp0\store1\version.txt
cd %~dp0\

:JM-checkvar
REM Set variable read from the source folder (revision.txt)
ECHO    ---------------------------------------
ECHO      set some settings
ECHO    ---------------------------------------
ECHO.
cd %~dp0\store1\
REM set version variable read from a file
set /p MAMEVERSION=<%~dp0\store1\version.txt
REM set revision variable read from a file
set /p Revision=<%~dp0\store1\revision.txt
cd %~dp0\

REM Main menu
:JM-MENU
cls
ECHO    ---------------------------------------
ECHO      ARCADE update and compiling V1.0a
ECHO    ---------------------------------------
ECHO         (c) 2016 mameguide.info
ECHO    ---------------------------------------
ECHO.
ECHO    [U] = Update local ARCADE source folder
ECHO    ---------------------------------------
ECHO    [C] = Compile ARCADE:
ECHO          - (V) Version		: %Compileversion%
ECHO          - (A) Architecture	: %CompileX%
ECHO          - (T) with Tools	: %Compiletoolsstring%
ECHO          - (F) add Folderstructure: %Addonfolder%
ECHO    ---------------------------------------
ECHO    (B) = Before compiling do a clean: %cleanmode%
ECHO    (P) = Batch Pause Mode           : %batchpause%
ECHO    ---------------------------------------
ECHO    [S] = Show last 200 commits in Notepad2
ECHO    ---------------------------------------
ECHO    [E] = Exit
ECHO    ---------------------------------------
ECHO.
ECHO    ---------------------------------------
ECHO    Note: When you having problems with 
ECHO          compiling and tools are not -/+ 
ECHO          generated - enable "clean" !!!!
ECHO          Compiling Source code is based
ECHO          on the ARCADE Repository
ECHO    ---------------------------------------
ECHO.
REM jump to the chosen part
CHOICE /C UCVATFBPSE /N /M "Command: "
IF ERRORLEVEL 10 GOTO JM-EXIT
IF ERRORLEVEL 9 GOTO JM-ShowLog
IF ERRORLEVEL 8 GOTO JM-PauseMode
IF ERRORLEVEL 7 GOTO JM-CleanMode
IF ERRORLEVEL 6 GOTO JM-Addonfolder
IF ERRORLEVEL 5 GOTO JM-Compiletools
IF ERRORLEVEL 4 GOTO JM-Compilearchitecture
IF ERRORLEVEL 3 GOTO JM-Compileversion
IF ERRORLEVEL 2 GOTO JM-Compiling
IF ERRORLEVEL 1 GOTO JM-UPDATESRC

REM Check the Pause Mode and set it to off or on
:JM-PauseMode
if [%batchpause%] EQU [off] goto JM-PauseMode1
if [%batchpause%] EQU [on] goto JM-PauseMode2
:JM-PauseMode1
set batchpause=on
GOTO JM-MENU
:JM-PauseMode2
set batchpause=off
GOTO JM-MENU

REM Check the Clean Mode and set it to off or on
:JM-CleanMode
if [%cleanmode%] EQU [off] goto JM-CleanMode1
if [%cleanmode%] EQU [on] goto JM-CleanMode2
:JM-CleanMode1
set cleanmode=on
GOTO JM-MENU
:JM-CleanMode2
set cleanmode=off
GOTO JM-MENU

REM Set the version to compile
:JM-Compileversion
if [%Compileversion%] EQU [MAME] goto JM-Compileversion1
if [%Compileversion%] EQU [ARCADE] goto JM-Compileversion2
:JM-Compileversion1
set Compileversion=ARCADE
GOTO JM-MENU
:JM-Compileversion2
set Compileversion=MAME
GOTO JM-MENU

REM Set the architecture to compile
:JM-Compilearchitecture
if [%CompileX%] EQU [x86] goto JM-Compilearchitecture1
if [%CompileX%] EQU [x64] goto JM-Compilearchitecture2

:JM-Compilearchitecture1
set CompileX=x64
Rem Check if it is a x86 architecture, in that case x64 is not possible
if defined PROGRAMFILES(X86) (
    set CompileX=x64
) else (
    set CompileX=x86
)
REM Check if the x64 compiling tools are installed
IF NOT EXIST ..\mingw64\ set CompileX=x86
GOTO JM-MENU

:JM-Compilearchitecture2
REM Check if the Compining-tools for x32 is available and set the value, if not available it can not be changed to x86
set CompileX=x86
IF NOT EXIST ..\mingw32\ set CompileX=x64
IF NOT EXIST ..\win32\config32.bat set CompileX=x64
GOTO JM-MENU

REM compile with tools or not
:JM-Compiletools
if [%Compiletools%] EQU [0] goto JM-Compiletools1
if [%Compiletools%] EQU [1] goto JM-Compiletools2
:JM-Compiletools1
set Compiletools=1
set Compiletoolsstring=on
set Compiletoolsstring2=with
ECHO    ------------------------------------------------
ECHO      enabling/disabling tools requires a clean up
ECHO    ------------------------------------------------
ECHO.
cd %~dp0\store1
make.exe clean
cd %~dp0\
GOTO JM-MENU
:JM-Compiletools2
set Compiletools=0
set Compiletoolsstring=off
set Compiletoolsstring2=without
ECHO    ------------------------------------------------
ECHO      enabling/disabling tools requires a clean up
ECHO    ------------------------------------------------
ECHO.
cd %~dp0\store1
make.exe clean
cd %~dp0\
GOTO JM-MENU

:JM-Addonfolder
REM Check the addonfolder value if it i off or on
if [%Addonfolder%] EQU [off] goto JM-Addonfolder1
if [%Addonfolder%] EQU [on] goto JM-Addonfolder2
:JM-Addonfolder1
set Addonfolder=on
GOTO JM-MENU
:JM-Addonfolder2
set Addonfolder=off
GOTO JM-MENU

:JM-UPDATESRC
cls
ECHO    ---------------------------------------
ECHO      Updating local ARCADE tool folder
ECHO    ---------------------------------------
ECHO.
cd ..
pacman -Sy
pacman -S bash pacman msys2-runtime --noconfirm --needed
pacman -Su --noconfirm
cd %~dp0\store1
ECHO.
ECHO    ---------------------------------------
ECHO      Updating local ARCADE source folder
ECHO    ---------------------------------------
ECHO.
REM download the update from the repository
git.exe pull -v --no-rebase --progress "origin"
REM Read the Build number from the file
set /p Build=<%~dp0\store1\.git\refs\heads\master
REM Ask the repository with the build number what revision number this is
git rev-list %Build% --count >%~dp0\store1\revision.txt
set /p Revision=<%~dp0\store1\revision.txt
REM read the tag from the repositiory into a file
FINDSTR /C:"#define MAME_VERSION" %~dp0\store1\src\osd\winui\winui.h >version_raw.txt
set /p MAMEVERSION=<version_raw.txt
set MAMEVERSION=%MAMEVERSION:~23,-1%
echo.%MAMEVERSION%>version.txt
set /p MAMEVERSION=<%~dp0\store1\version.txt
ECHO.
ECHO    ---------------------------------------
ECHO      Version	:  %MAMEVERSION%
ECHO      Build	:  %Build%
ECHO      Revision	:  %Revision%
ECHO    ---------------------------------------
ECHO.
cd %~dp0\
REM do a pause if it is set
if [%batchpause%] EQU [on] pause
GOTO JM-MENU

:JM-Compiling
cls
ECHO    ------------------------------------------
ECHO      Compiling now %Compileversion% %CompileX% %Compiletoolsstring2% tools
ECHO    ------------------------------------------
ECHO.
REM Check if x32/x64 version of the compiling tools is installed
IF NOT EXIST ..\win32\config64.bat GOTO JM-Folder1
IF NOT EXIST ..\mingw32\ GOTO JM-Folder1
cd ..\win32\
if [%CompileX%] EQU [x64] @call config64.bat
if [%CompileX%] EQU [x86] @call config32.bat
:JM-Folder1
cd %~dp0\store1\
REM Check if folder ARCADE_builds exists or not
IF EXIST %~dp0\_ARCADE\%Compileversion%\ GOTO JM-Folder2
ECHO    ------------------------------------------
ECHO      creating folder  %~dp0\_ARCADE\%Compileversion%\
ECHO    ------------------------------------------
ECHO.
mkdir %~dp0\_ARCADE\%Compileversion%\
:JM-Folder2
REM first do a cleanup and then compiling with that options
if [%cleanmode%] EQU [on] make.exe clean
if [%Compiletools%] EQU [0] goto JM-Folder3
REM compile the right version with tools  (OSD=winui,  ARCHITECTURE=_%CompileX%)
if [%Compileversion%] EQU [MAME] make.exe -%Compilecpu% TOOLS=1 OSD=winui
if [%Compileversion%] EQU [ARCADE] make.exe -%Compilecpu% TOOLS=1 SUBTARGET=arcade OSD=winui
goto JM-Folder4
:JM-Folder3
REM compile the right version without tools
if [%Compileversion%] EQU [MAME] make.exe -%Compilecpu% OSD=winui
if [%Compileversion%] EQU [ARCADE] make.exe -%Compilecpu% SUBTARGET=arcade OSD=winui
:JM-Folder4
cd %~dp0\
REM set foldername from the version and revision that is set from the source folder
set foldername=%Mameversion%r%Revision%%CompileX%
REM check if folder does exists -> if yes delete it with content
IF EXIST %~dp0\_ARCADE\%Compileversion%\%foldername% rmdir /Q /S %~dp0\_ARCADE\%Compileversion%\%foldername%
REM create a foldername with that information 
mkdir %~dp0\_ARCADE\%Compileversion%\%foldername%
REM Move all executables from the source folder to this new subfolder
ECHO    ------------------------------------------
ECHO      moving created files to %~dp0\_ARCADE\%Compileversion%\%foldername%\
ECHO    ------------------------------------------
ECHO.
move /Y "%~dp0\store1\*.exe" "%~dp0\_ARCADE\%Compileversion%\%foldername%\"
REM copy addon folder/files when enabled
if [%Addonfolder%] EQU [off] goto JM-Folder5
ECHO    ------------------------------------------
ECHO      copying now addon folder structure 
ECHO    ------------------------------------------
ECHO.
xcopy /E "%~dp0\store1\artwork" "%~dp0\_ARCADE\%Compileversion%\%foldername%\artwork\" >NUL
xcopy /E "%~dp0\store1\bgfx" "%~dp0\_ARCADE\%Compileversion%\%foldername%\bgfx\" >NUL
xcopy /E "%~dp0\store1\docs" "%~dp0\_ARCADE\%Compileversion%\%foldername%\docs\" >NUL
xcopy /E "%~dp0\store1\hash" "%~dp0\_ARCADE\%Compileversion%\%foldername%\hash\" >NUL
xcopy /E "%~dp0\store1\hlsl" "%~dp0\_ARCADE\%Compileversion%\%foldername%\hlsl\" >NUL
xcopy /E "%~dp0\store1\ini" "%~dp0\_ARCADE\%Compileversion%\%foldername%\ini\" >NUL
xcopy /E "%~dp0\store1\language" "%~dp0\_ARCADE\%Compileversion%\%foldername%\language\" >NUL
xcopy /E "%~dp0\store1\nl_examples" "%~dp0\_ARCADE\%Compileversion%\%foldername%\nl_examples\" >NUL
xcopy /E "%~dp0\store1\plugins" "%~dp0\_ARCADE\%Compileversion%\%foldername%\plugins\" >NUL
mkdir "%~dp0\_ARCADE\%Compileversion%\%foldername%\roms" >NUL
xcopy /E "%~dp0\store1\samples" "%~dp0\_ARCADE\%Compileversion%\%foldername%\samples\" >NUL
:JM-Folder5
REM do a pause if it is set
if [%batchpause%] EQU [on] pause
GOTO JM-MENU

:JM-ShowLog
cls
cd %~dp0\store1\
REM Read the Build number from the local file
set /p Build=<%~dp0\store1\.git\refs\heads\master
REM Read the version number from the local file
set /p Revision=<%~dp0\store1\revision.txt
REM read the last entry from the local file
set /p MAMEVERSION=<%~dp0\store1\version.txt
REM read the commit number from the remote source (master) and write it to a file
git ls-remote https://github.com/Robbbert/store1.git master >%~dp0\store1\remotebuild.txt
REM Set variable from the written file
set /P Remotebuild=<%~dp0\store1\remotebuild.txt
REM cut everything above the 40 position nad set it again as variable
set Remotebuild=%Remotebuild:~0,40%
rem git rev-list %Remotebuild% --count >revisionremote.txt
rem set /p Revisionremote=<revisionremote.txt
ECHO    ---------------------------------------
ECHO     Show last 200 commits in Notepad2
ECHO    ---------------------------------------
ECHO	 Local source version:
ECHO.
ECHO     Version	:  %MAMEVERSION%
ECHO     Build	:  %Build%
ECHO     Revision	:  %Revision%
ECHO    ---------------------------------------
ECHO.
REM Check if remote source has the same commit number than local
If %Build%==%Remotebuild% GOTO JM-SHOWLOG1
ECHO     local source is not up-to-date
ECHO     Remote source code is at %Remotebuild%
ECHO    ---------------------------------------
ECHO.
GOTO JM-SHOWLOG2
:JM-SHOWLOG1
ECHO     local source is up-to-date with remote
ECHO    ---------------------------------------
ECHO.
:JM-SHOWLOG2
REM write the last 200 commits to a file
git.exe log -200 >%~dp0\store1\log.txt
cd %~dp0\
REM do a pause if it is set
if [%batchpause%] EQU [on] pause
REM Open Notepad2 with the log-file
%~dp0\notepad2.exe %~dp0\store1\log.txt
REM Remove temporary files (not needed for other tasks)
del %~dp0\store1\log.txt
del %~dp0\store1\remotebuild.txt
del %~dp0\store1\revisionremote.txt
GOTO JM-MENU

:JM-EXIT